/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _LASER_H
#define _LASER_H

#include "calc.h"
#include "direction.h"
#include "Sprite.h"

#define LASER_SPEED 5
#define HP_DIST_FROM_TIP 2
#define LASER_LIFE 4

typedef enum laser_type {
		PLAYER,
		//CHARGE,
		ENEMY
} laser_type;

class Laser : public Sprite {
private:
	int dir;
	int vel;
	int life;
	enum {
		PRESENT,
		VANISHING,
		VANISHED,
		APPEARING,
		TRANSIT
	} vanishing;
	int appear_x;
	int appear_y;
	int vanish_x;
	int vanish_y;
	enum laser_type type;

public:
	/** @brief Create a laser whose front is centered at (x, y).
	 *
	 *  @param x The initial x-position of the laser.
	 *  @param y The initial y-position of the laser.
	 *  @param dir The direction of the laser: NORTH, EAST, SOUTH, or WEST.
	 *  @param vel The magnitude of the laser's velocity.
	 *  @param type The type of the laser.
	 */
	Laser(int x, int y, int dir, int vel, int life, laser_type type);

	/** @brief Create a laser with a specific hot point.
	 *
	 */
	Laser(point c, int dir, int vel, int life, laser_type type);

	/** @brief Changes the direction of the laser.
	 */
	//void changeDirection(int dir);

	/** @brief Return the hot point of the laser. This is the point
	 *         where the laser will collide with objects.
	 *
	 *  Returns (-1, -1) if the laser is appearing, vanishing, or vanished.
	 */
	point getHotPoint();

	/** @brief Return the velocity of the laser.
	 *
	 */
	int getVelocity();

	/** @brief Return the direction of the laser.
	 */
	int getDirection();

	/** @brief Return the life left in this laser.
	 */
	int getLife();

	/** @brief Return the type of the laser.
	 */
	laser_type getType();

	/** @brief Notify the laser that it is vanishing.
	 *
	 *  The head of the laser when this function is called becomes the point
	 *  at which the laser vanishes. That is, the laser disappears bit by bit
	 *  as it travels through that point.
	 */
	void vanish();

	/** @brief Returns true if the laser is vanishing.
	 */
	bool isVanishing();

	/** @brief Returns true if the laser has finished the vanishing process.
	 */
	bool vanished();

	virtual void draw();
	virtual void move();
};

#endif /* _LASER_H */
