/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Level.h"
#include "calc.h"
#include "debug.h"
#include <fstream>
#include <stdio.h>
#include <iostream>
using namespace std;

//Ships
#include "Crate.h"
#include "Enemy.h"

Level::Level(int num, SDL_Surface* bg, int player) : lvlnum(num),
		background(bg), num_enemies(0), plasers(0), state(GO) {
	this->player = new Player(0, 0, MAX_SHOTS, player, &addLaserToCurrentLevel);

	char c[strlen(LEVELDIR) + strlen(LEVELTXT) + 3];
	// 1 for the null terminator, 2 to allow numbers of up to 4 digits.
	//allows for up to 9999 levels.
	sprintf(c, LEVELDIR LEVELTXT, num);
	db_log(DB_LOAD, "Loading %s...\n", c);
	this->loadFile(c);
}

Level::~Level() {
	list<Ship*>::iterator enemy;
	for (enemy = enemies.begin(); enemy != enemies.end(); enemy++) {
		delete *enemy;
	}
	delete player;
}

#define WALL "Wall"
#define SHIPAREA "ShipArea"
#define SHIP "Ship"
#define CRATE "Crate"
#define FORWARD_SHOOTER "EnemyForward"
#define DUAL_SHOOTER "EnemyDual"

int getDirection(string dir) {
	if (dir.compare("UP") == 0) return NORTH;
	else if (dir.compare("RIGHT") == 0) return EAST;
	else if (dir.compare("LEFT") == 0) return WEST;
	else if (dir.compare("DOWN") == 0) return SOUTH;
	else return -1;
}

list<point> getPath(string pointlist) {
	list<point> path;
	point c;
	string t = pointlist;
	int next;
	while (1) {
		sscanf(t.c_str(), "(%d, %d)", &c.x, &c.y);
		path.push_back(c);
		next = t.find_first_of('(', 1);
		if (next < 0) break;
		else {
			t = t.substr(next);
		}
	}
	return path;
}

void Level::loadFile(const char *filename) {
	string line;
	ifstream f (filename);
	if (f.is_open()) {
		this->player->clearBounds();
		while (! f.eof()) {
			getline(f, line);
			int i = line.find_first_of('(');
			if (i < 0) continue;
			string s = line.substr(0, i),
				t = line.substr(i);
			if (s.compare(WALL) == 0) {
				int a, b, c, d;
				sscanf(t.c_str(), "(%d, %d) (%d, %d)", &a, &b, &c, &d);
				db_log(DB_LOAD, "Creating Wall (%d, %d)(%d, %d)\n",
						a, b, c, d);
				Wall* w = new Wall(a, b, c, d);
				this->addWall(*w);
				delete w;
			} else if (s.compare(SHIPAREA) == 0) {
				int a, b, c, d;
				sscanf(t.c_str(), "(%d, %d) (%d, %d)", &a, &b, &c, &d);
				db_log(DB_LOAD, "Adding bounds (%d, %d)(%d, %d)\n",
						a, b, c, d);
				this->player->addBounds(a, b, c, d);
			} else if (s.compare(SHIP) == 0) {
				int a, b;
				sscanf(t.c_str(), "(%d, %d)", &a, &b);
				db_log(DB_LOAD, "Setting player position (%d, %d)\n", a, b);
				this->player->setPos(a, b);
			} else if (s.compare(CRATE) == 0) {
				int a, b;
				sscanf(t.c_str(), "(%d, %d)", &a, &b);
				db_log(DB_LOAD, "Adding Crate (%d, %d)\n", a, b);
				Crate* c = new Crate(a, b);
				this->addEnemy(c);
			} else if (s.compare(FORWARD_SHOOTER) == 0) {
				int a, b;
				sscanf(t.c_str(), "(%d, %d)", &a, &b);
				int dpos = t.find_first_of(')') + 1;
				int next = t.find_first_of('(', dpos);
				// From beginning of dir text to (but not including) '('
				string d = t.substr(dpos, next - dpos);
				int dir = getDirection(d);
				list<point> path;
				if (next < 0) {
					point c;
					c.x = a;
					c.y = b;
					path.push_back(c);
				} else {
					path = getPath(t.substr(next));
				}
				db_log(DB_LOAD, "Adding FShooter (%d, %d) dir %d\n", a, b, dir);
				FShooter* f = new FShooter(a, b, dir, path, FSHOOTER_VEL,
						FSHOOTER_RATE, FSHOOTER_OFFSET,
						&addLaserToCurrentLevel);
				this->addEnemy(f);
			} else if (s.compare(DUAL_SHOOTER) == 0) {
				int a, b;
				sscanf(t.c_str(), "(%d, %d)", &a, &b);
				int dpos = t.find_first_of(')') + 1;
				int next = t.find_first_of('(', dpos);
				// From beginning of dir text to (but not including) '('
				string d = t.substr(dpos, next - dpos);
				int dir = getDirection(d);
				list<point> path;
				if (next < 0) {
					point c;
					c.x = a;
					c.y = b;
					path.push_back(c);
				} else {
					path = getPath(t.substr(next));
				}
				db_log(DB_LOAD, "Adding DShooter (%d, %d) dir %d\n", a, b, dir);
				DShooter* f = new DShooter(a, b, dir, path, DSHOOTER_VEL,
						DSHOOTER_RATE, DSHOOTER_OFFSET,
						&addLaserToCurrentLevel);
				this->addEnemy(f);
			} else {
				db_log(DB_LOAD, "Unknown tag: %s\n", s.c_str());
			}
		}
		f.close();
	} else {
		cerr << "Cannot open file: " << filename << endl;
	}
}

void Level::addLaser(Laser l) {
	lasers.push_back(l);
	if (l.getType() == PLAYER) {
		plasers++;
		if (plasers > player->getMaxShots()) {
			list<Laser>::iterator lit;
			for (lit = lasers.begin(); lit != lasers.end(); lit++) {
				if (! lit->isVanishing() && lit->getType() == PLAYER) {
					lit->vanish();
					plasers--;
					break;
				}
			}
		}
	}
}

void Level::addWall(Wall w) {
	walls.push_back(w);
}

void Level::addEnemy(Ship* s) {
	enemies.push_back(s);
	num_enemies++;
}

void Level::checklasers() {
	list<Laser>::iterator laser;
	list<Ship*>::iterator enemy;
	list<Wall>::iterator wall;
	Laser* n;
	point c;
	int x, y;

	for (laser = lasers.begin(); laser != lasers.end(); laser++) {
		c = laser->getHotPoint();
		x = c.x, y = c.y;
		for (wall = walls.begin(); wall != walls.end(); wall++) {
			if (wall->doesCollide(x, y)) {
				laser->vanish();
				/* We need to be careful that the laser won't recollide
				 * with the same wall. If we insert it in front of this laser,
				 * we won't check it.
				 */
				int life = laser->getLife();
				int dir = laser->getDirection();
				int dir2 = reflect(dir, wall->getDirection());
				if (difference(dir, dir2) == INVERSE) life--;
				if (life > 0) {
					n = new Laser(c, dir2, laser->getVelocity(), life,
						laser->getType());
					lasers.insert(laser, *n);
					delete n;
				} else if (laser->getType() == PLAYER) {
					plasers--;
				}
				break;
			}
		}
		if (laser->getType() != ENEMY) {
			for (enemy = enemies.begin(); enemy != enemies.end(); enemy++) {
				if ((*enemy)->doesCollide(x, y)) {
					if (laser->getType() == PLAYER) {
						laser->vanish();
						plasers--;
					}
					(*enemy)->hit();
					break;
				}
			}
		}
		if (player->doesCollide(x, y)) {
			laser->vanish();
			player->hit();
		}
	}
}

void Level::draw() {
	list<Laser>::reverse_iterator laser;
	list<Ship*>::iterator enemy;
	list<Wall>::iterator wall;

	DrawImage(background, 0, 0);
	player->draw();
	for (enemy = enemies.begin(); enemy != enemies.end();
			(*enemy)->isDead() ? (num_enemies--, enemy = enemies.erase(enemy))
					: enemy++) {
		(*enemy)->draw();
	}
	for (laser = lasers.rbegin(); laser != lasers.rend(); laser++) {
		laser->draw();
	}
	for (wall = walls.begin(); wall != walls.end(); wall++) {
		wall->draw();
	}
	if (num_enemies == 0) {
		state = WIN;
	}
	if (player->isDead()) {
		state = LOSE;
	}
}

Player* Level::getPlayer() {
	return player;
}

void Level::step() {
	if (state != GO) return;
	list<Laser>::iterator laser;
	list<Ship*>::iterator enemy;
	list<Wall>::iterator wall;

	player->move();
	for (enemy = enemies.begin(); enemy != enemies.end(); enemy++) {
		(*enemy)->move();
	}
	for (laser = lasers.begin(); laser != lasers.end();
			/* This will remove dead lasers. */
			laser->vanished() ? (laser = lasers.erase(laser)) : laser++) {
		laser->move();
	}

	this->checklasers();
}

bool Level::lost() {
	return state == LOSE;
}

bool Level::won() {
	return state == WIN;
}

Level *currentlevel;

void addLaserToCurrentLevel(Laser l) {
	if (currentlevel != NULL) {
		currentlevel->addLaser(l);
	}
}

void changeLevel(int num, SDL_Surface* bg, int player) {
	Level *l = new Level(num, bg, player);
	if (currentlevel != NULL) {
		Level *n = currentlevel;
		currentlevel = l;
		delete n;
	} else {
		currentlevel = l;
	}
}

Level* getCurrentLevel() {
	return currentlevel;
}
