/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _LEVEL_H
#define _LEVEL_H

#include <stdlib.h>
#include <iostream>
#include <list>
#include "graphics.h"
#include "Ship.h"
#include "Player.h"
#include "Laser.h"
#include "Wall.h"

#define LEVELDIR "levels/"
#define LEVELTXT "level%d.txt"
#define NUM_LEVELS 35

using namespace std;

class Level {
private:
	int lvlnum;
	int num_enemies;
	int plasers;
	list<Ship*> enemies;
	list<Laser> lasers;
	list<Wall> walls;
	Player* player;
	SDL_Surface* background;
	enum {
		GO,
		LOSE,
		WIN
	} state;

	void checklasers();
	void loadFile(const char *filename);

public:
	Level(int num, SDL_Surface *bg, int player);
	~Level();

	static const char* getLevelFile(int n);

	/** @brief Adds a wall to the level.
	 *
	 *  The level contains a copy of the wall passed.
	 */
	void addWall(Wall w);

	/** @brief Adds a laser to the level.
	 *
	 *  The level contains a copy of the laser passed.
	 */
	void addLaser(Laser l);

	/** @brief Adds an enemy ship to the level.
	 *
	 *  The level contains a pointer to the Ship passed.
	 *  The level will delete the object when the level is deleted.
	 */
	void addEnemy(Ship* s);

	/** @brief Returns a pointer to the player for this level.
	 *
	 *  This pointer is only valid as long as this level exists.
	 *  The player is deleted when the level is deleted.
	 */
	Player* getPlayer();

	void draw();
	void step();

	bool won();
	bool lost();
};

void addLaserToCurrentLevel(Laser l);

void changeLevel(int num, SDL_Surface* bg, int player);

Level* getCurrentLevel();

#endif /* _LEVEL_H */
