/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Player.h"
#include "direction.h"
#include "graphics.h"
#include <algorithm>

static SDL_Surface* findImage(int i) {
	switch(i) {
	case 2: return GetImage(IMG_PLAYER2);
	case 3: return GetImage(IMG_PLAYER3);
	default: return GetImage(IMG_PLAYER);
	}
}

Player::Player(int x, int y, int shots, int image,
		void (*laser_callback)(Laser))
			: maxshots(shots), Ship(NORTH, x, y,
				findImage(image), laser_callback) {
	//setBounds(bounds_x1, bounds_y1, bounds_x2, bounds_y2);
}

void Player::clearBounds() {
	bounds.clear();
}

void Player::addBounds(int bounds_x1, int bounds_y1, int bounds_x2,
		int bounds_y2) {
	int bounds_n, bounds_e, bounds_w, bounds_s;
	bound b;
	if (bounds_x1 < bounds_x2) {
		bounds_w = bounds_x1;
		bounds_e = bounds_x2;
	} else {
		bounds_w = bounds_x2;
		bounds_e = bounds_x1;
	}
	if (bounds_y1 < bounds_y2) {
		bounds_n = bounds_y1;
		bounds_s = bounds_y2;
	} else {
		bounds_n = bounds_y2;
		bounds_s = bounds_y1;
	}
	b.bounds_n = bounds_n;
	b.bounds_e = bounds_e;
	b.bounds_s = bounds_s;
	b.bounds_w = bounds_w;
	bounds.push_back(b);
}

void Player::setPos(int x, int y) {
	pos_x = x;
	pos_y = y;
}

int Player::getMaxShots() {
	return maxshots;
}

#define HITBOX_U 20
#define HITBOX_L 15
#define HITBOX_R 35
#define HITBOX_D 40

bool Player::doesCollide(int x, int y) {
	return (x >= pos_x + HITBOX_L && x <= pos_x + HITBOX_R &&
			y >= pos_y + HITBOX_U && y <= pos_y + HITBOX_D);
}

void Player::fireLaser() {
	if (state != ALIVE) return;
	point c = center_side(NORTH);
	Laser* laser = new Laser(c.x, c.y, NORTH, LASER_SPEED, LASER_LIFE, PLAYER);
	if (firedLaser != NULL) {
		firedLaser(*laser);
	}
}

bool Player::inbounds(bound b, int px, int py) {
	return (px + image->w < b.bounds_e && px > b.bounds_w)
		&& (py + image->h < b.bounds_s && py > b.bounds_n);
}

void Player::move() {
	if (state != ALIVE) return;
	int px = pos_x + vel_x;
	int py = pos_y + vel_y;
	list<bound>::iterator b;
	for (b = bounds.begin(); b != bounds.end(); b++) {
		if (this->inbounds(*b, px, py)) {
			pos_x = px;
			pos_y = py;
			return;
		}
	}
}

void Player::hit() {
	this->Ship::hit();
}

void drawBounds(bound b) {
	SDL_Surface* V = GetImage(IMG_BOUNDS_V);
	SDL_Surface* H = GetImage(IMG_BOUNDS_H);
	for (int starth = b.bounds_n, endh = b.bounds_s,
			diff = min(endh - starth, V->h);
				endh - starth > 0;
				starth += diff, diff = min(endh - starth, V->h)) {
		DrawPartialImage(V, b.bounds_w, starth, 0, 0, V->w, diff);
		DrawPartialImage(V, b.bounds_e - V->w, starth, 0, 0, V->w, diff);
	}
	for (int startw = b.bounds_w, endw = b.bounds_e,
			diff = min(endw - startw, H->w);
				endw - startw > 0;
				startw += diff, diff = min(endw - startw, H->w)) {
		DrawPartialImage(H, startw, b.bounds_n, 0, 0, diff, H->h);
		DrawPartialImage(H, startw, b.bounds_s - H->h, 0, 0, diff, H->h);
	}
}

void Player::draw() {
	/* Draw the bounding box and the ship. */

	// Draw the ship.
	this->Ship::draw();
	for (list<bound>::iterator b = bounds.begin(); b != bounds.end(); b++) {
		drawBounds(*b);
	}
}
