/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _SHIP_H
#define _SHIP_H

#include "direction.h"
#include "Sprite.h"
#include "Laser.h"
#include "graphics.h"

class Ship : public Sprite {
protected:
	int direction;
	void (*firedLaser)(Laser);
	int countdown;
	enum {
		ALIVE,
		DEAD,
		DYING
	} state;
	point center_side(int dir);

public:
	/** @brief Constructs a Ship object with the specified direction,
	 *         at the specified point, with the specified image,
	 *         and the specified laser callback (called when the ship
	 *         fires a laser).
	 */
	Ship(int dir, int x, int y, SDL_Surface* image,
			void (*laser_callback)(Laser));

	/** @brief Returns this ship's direction.
	 */
	int getDirection();

	/** @brief Returns true if the ship is dead.
	 */
	bool isDead();

	/** @brief Whether or not that point intersects this ship.
	 */
	virtual bool doesCollide(int x, int y);

	/** @brief The ship has been hit by a laser.
	 */
	virtual void hit();

	/** @brief The ship fires its laser(s).
	 */
	virtual void fireLaser();

	virtual void draw();
};

#endif /* _SHIP_H */
