/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Sprite.h"
#include "graphics.h"

Sprite::Sprite(int posx, int posy, SDL_Surface* image) : pos_x(posx),
		pos_y(posy), vel_x(0), vel_y(0), image(image) {
	static unsigned int nextID = 0;
	this->id = nextID++;
}

int Sprite::getXPosition() {
	return pos_x;
}

int Sprite::getYPosition() {
	return pos_y;
}

int Sprite::getXVelocity() {
	return vel_x;
}

int Sprite::getYVelocity() {
	return vel_y;
}

void Sprite::setXVelocity(int vel) {
	vel_x = vel;
}

void Sprite::setYVelocity(int vel) {
	vel_y = vel;
}

SDL_Surface* Sprite::getImage() {
	return image;
}

void Sprite::step(int dir) {
	/* This is highly dependent on what we consider to be the point (0,0).
	 * If we use the upper left corner as (0,0) (or (1,1)), then
	 * going NORTH is -, EAST is +, SOUTH is +, WEST is -.
	 */
	switch(dir) {
	case NORTH: pos_y--; break;
	case NORTHEAST: pos_x++; pos_y--; break;
	case EAST:  pos_x++; break;
	case SOUTHEAST: pos_x++; pos_y++; break;
	case SOUTH: pos_y++; break;
	case SOUTHWEST: pos_x--; pos_y++; break;
	case WEST:  pos_x--; break;
	case NORTHWEST: pos_x--; pos_y--; break;
	}
}

void Sprite::move() {
	pos_x += vel_x;
	pos_y += vel_y;
}

void Sprite::draw() {
	DrawImage(this->image, this->pos_x, this->pos_y);
}
