/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <stdlib.h>
#include <time.h>
#include "SDL.h"
#include "graphics.h"
#include "debug.h"
#include "Wall.h"
#include "Level.h"
#include "Player.h"
#include "direction.h"
#include <algorithm>

using namespace std;

#define HIGHLIGHT_X1 345
#define HIGHLIGHT_Y1 304
#define HIGHLIGHT_X2 346
#define HIGHLIGHT_Y2 473

#define SELECT_Y 623
#define SELECT_X1 30
#define SELECT_X2 400
#define SELECT_X3 767

#define CLOCKS_PER_ITER 10000
#define CLOCKS_PER_MS (CLOCKS_PER_SEC / 1000)

SDL_Surface *screen; //back buffer

/* Game state */
enum {
	TITLE,
	INSTRUCTIONS,
	SHIP_SELECT,
	GAME,
	LEVEL_LOST,
	LEVEL_WON,
	GAME_OVER,
	CREDITS,
	EXIT
};

/*
 http://cc.byexamples.com/20070525/nanosleep-is-better-than-sleep-and-usleep/
*/
int msleep(unsigned long milisec) {
    struct timespec req={0};
    time_t sec=(int)(milisec/1000);
    milisec=milisec-(sec*1000);
    req.tv_sec=sec;
    req.tv_nsec=milisec*1000000L;
    while(nanosleep(&req,&req)==-1)
         continue;
    return 1;
}

#define MAX_LIVES 4

int level = 1;
int maxlevel = 1;
int lives = MAX_LIVES;
int p = 1;
SDL_Surface *bg;

int handleTitle() {
	int sel = 0;

	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
					return EXIT;
				case SDLK_UP:
				case SDLK_DOWN:
					sel = 1 - sel;
					break;
				case SDLK_SPACE:
				case SDLK_RETURN:
					if (sel == 0) {
						level = 1;
						lives = MAX_LIVES;
						return INSTRUCTIONS;
					} else {
						return CREDITS;
					}
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
		
			Slock(screen);
			DrawImage(GetImage(IMG_MENU), 0, 0);
			if (sel == 0) {
				DrawImage(GetImage(IMG_MENU_BOX),
					HIGHLIGHT_X1, HIGHLIGHT_Y1);
			} else {
				DrawImage(GetImage(IMG_MENU_BOX),
					HIGHLIGHT_X2, HIGHLIGHT_Y2);
			}
			Sulock(screen);
			SDL_Flip(screen);
		}
	}
}

int handleInstructions() {
	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
					return TITLE;
				case SDLK_SPACE:
				case SDLK_RETURN:
					return SHIP_SELECT;
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
			Slock(screen);
			DrawImage(GetImage(IMG_INSTRUCTIONS), 0, 0);
			Sulock(screen);
			SDL_Flip(screen);
		}
	}
}

int handleShipSelect() {
	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
					return TITLE;
				case SDLK_RIGHT:
					if (p < 3) p++;
					break;
				case SDLK_LEFT:
					if (p > 1) p--;
					break;
				case SDLK_SPACE:
				case SDLK_RETURN:
					return GAME;
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
			Slock(screen);
			DrawImage(GetImage(IMG_SHIP_SELECT), 0, 0);
			if (p == 2) {
				DrawImage(GetImage(IMG_SHIP_SELECT_BAR),
					SELECT_X2, SELECT_Y);
			} else if (p == 3) {
				DrawImage(GetImage(IMG_SHIP_SELECT_BAR),
					SELECT_X3, SELECT_Y);
			} else {
				DrawImage(GetImage(IMG_SHIP_SELECT_BAR),
					SELECT_X1, SELECT_Y);
			}
			Sulock(screen);
			SDL_Flip(screen);
		}
	}
}

int handleGame() {
	Level *current;
	Player *player;

	changeLevel(level, bg, p);
	current = getCurrentLevel();
	player = current->getPlayer();

	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYUP:
				switch(event.key.keysym.sym) {
				/* If we release a key, set velocity to 0 only if
				* we were moving in that direction.
				*/
				case SDLK_LEFT:
					if (player->getXVelocity() < 0) {
						player->setXVelocity(0);
					}
					break;
				case SDLK_RIGHT:
					if (player->getXVelocity() > 0) {
						player->setXVelocity(0);
					}
					break;
				case SDLK_UP:
					if (player->getYVelocity() < 0) {
						player->setYVelocity(0);
					}
					break;
				case SDLK_DOWN:
					if (player->getYVelocity() > 0) {
						player->setYVelocity(0);
					}
					break;
				}
				break;
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
				case SDLK_q:
					return TITLE;
				case SDLK_LEFT:
					player->setXVelocity(-PLAYER_VEL);
					break;
				case SDLK_RIGHT:
					player->setXVelocity(PLAYER_VEL);
					break;
				case SDLK_UP:
					player->setYVelocity(-PLAYER_VEL);
					break;
				case SDLK_DOWN:
					player->setYVelocity(PLAYER_VEL);
					break;
				case SDLK_SPACE:
					/* Create a laser at the nose of the ship. */
					player->fireLaser();
					break;
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
		}

		static clock_t last = 0, next;
		/* Enforce a delay so that the game speed runs the same. */
		next = clock();
		if (next - last < CLOCKS_PER_ITER) {
			msleep((CLOCKS_PER_ITER - (next - last)) / CLOCKS_PER_MS);
		}
		last = next;
		current->step();
		Slock(screen);
		current->draw();
		SDL_Surface* ship = player->getImage();
		for (int i = 1; i < lives; i++) {
			DrawImage(ship, SCREEN_WIDTH - i * (ship->w),
				SCREEN_HEIGHT - ship->h);
		}
		Sulock(screen);
		SDL_Flip(screen);

		if (current->won()) {
			return LEVEL_WON;
		} else if (current->lost()) {
			lives--;
			if (lives > 0) {
				return LEVEL_LOST;
			} else {
				return GAME_OVER;
			}
		}
	}
}

int handleLevelLost() {
	/* Only need to draw the image once. */
	Slock(screen);
	DrawImage(GetImage(IMG_LEVEL_LOSE), 0, 0);
	Sulock(screen);
	SDL_Flip(screen);

	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
					return TITLE;
				case SDLK_SPACE:
				case SDLK_RETURN:
					return GAME;
				case SDLK_s:
					return SHIP_SELECT;
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
		}
	}
}

int handleLevelWon() {
	/* Only need to draw the image once. */
	Slock(screen);
	DrawImage(GetImage(IMG_LEVEL_WIN), 0, 0);
	Sulock(screen);
	SDL_Flip(screen);

	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
					return TITLE;
				case SDLK_SPACE:
				case SDLK_RETURN:
					if (level == NUM_LEVELS) {
						return TITLE;
					}
					maxlevel = max(maxlevel, level + 1);
					level++;
					return GAME;
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
		}
	}
}

int handleGameOver() {
	/* Only need to draw the image once. */
	Slock(screen);
	DrawImage(GetImage(IMG_GAME_OVER), 0, 0);
	Sulock(screen);
	SDL_Flip(screen);

	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
				case SDLK_SPACE:
				case SDLK_RETURN:
					return TITLE;
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
		}
	}
}

int handleCredits() {
	/* Only need to draw the image once. */
	Slock(screen);
	DrawImage(GetImage(IMG_CREDITS), 0, 0);
	Sulock(screen);
	SDL_Flip(screen);

	while (1) {
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
				case SDLK_SPACE:
				case SDLK_RETURN:
					return TITLE;
				}
				break;
			case SDL_QUIT:
				return EXIT;
			}
		}
	}
}

int main(int argc, char *argv[]) {
	db_log(DB_START, "Initializing SDL.\n");
	/* Initializes Audio and the CDROM, add SDL_INIT_VIDEO for Video */
	if(SDL_Init(SDL_INIT_AUDIO | SDL_INIT_CDROM)< 0) {
		cerr <<"Could not initialize SDL:" << SDL_GetError() << endl;
		SDL_Quit();
		return 1;
	} else {
		db_log(DB_START, "Audio & CDROM initialized correctly\n");
		/* Trying to read number of CD devices on system */
		db_log(DB_START, "Drives available : %d\n", SDL_CDNumDrives());
		for(int i=0; i < SDL_CDNumDrives(); ++i) {
			db_log(DB_START, "Drive %d\"%s\"\n", i, SDL_CDName(i));
		}
	}
	atexit(SDL_Quit);

	//screen = SDL_SetVideoMode(640, 480, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
	screen = InitVideo(SDL_DOUBLEBUF | SDL_SWSURFACE);
	//initgraphics(screen);

	if ( screen == NULL ) {
		cerr << "Unable to set 640x480 video: " << SDL_GetError() << endl;
		return 1;
	}

	db_log(DB_IMAGE, "Initializing images...\n");
	if (!InitImages()) {
		cerr << "Unable to load images." << endl;
		return 1;
	}


	int state = TITLE;

	bg = GetImage(IMG_BG_SPACE);

	while (state != EXIT) {
		switch(state) {
		case TITLE:
			state = handleTitle();
			break;
		case INSTRUCTIONS:
			state = handleInstructions();
			break;
		case SHIP_SELECT:
			state = handleShipSelect();
			break;
		case GAME:
			state = handleGame();
			break;
		case LEVEL_LOST:
			state = handleLevelLost();
			break;
		case LEVEL_WON:
			state = handleLevelWon();
			break;
		case GAME_OVER:
			state = handleGameOver();
			break;
		case CREDITS:
			state = handleCredits();
			break;
		}
	}

	UnloadImages();
}
