/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _DIRECTION_H
#define _DIRECTION_H

enum {
	NORTH	  = 0,
	NORTHEAST = 45,
	EAST	  = 90,
	SOUTHEAST = 135,
	SOUTH	  = 180,
	SOUTHWEST = 225,
	WEST	  = 270,
	NORTHWEST = 315
};

#define RIGHT_ANGLE 90
#define HALF_CIRCLE 180
#define INVERSE 180
#define CIRC_INVERSE(x) (360 - (x))

#define MOD360(x) (((x) % 360 >= 0) ? ((x) % 360) : 360 - (-(x) % 360))

/** @brief Modifies a direction by rotating clockwise the specified
 *         number of degrees.
 *
 *  @param dir The direction to rotate.
 *  @param rotation Number of degrees to rotate.
 */
int rotate(int dir, int rotation);

/** @brief Calculates the difference between two directions.
 *
 *  @param dir1 The first direction.
 *  @param dir2 The second direction.
 *  @return The number of degrees necessary to rotate the first direction
 *          to reach the second, between -179 and 180. (Negative means CCW.)
 */
int difference(int dir1, int dir2);

/** @brief Modifies a direction by reflecting against a surface
 *         whose normal is specified.
 *
 *  Any incoming direction that is not within 90 degrees of the normal
 *  is not affected.
 *
 *  @param dir The direction being reflected.
 *  @param normal The normal to the reflective surface.
 */
int reflect(int dir, int normal);

#endif /* DIRECTION_H */
