/***************************************************************************
 *   Copyright (C) 2009 by Ben Wolf   *
 *   bswolf@cs.cmu.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "graphics.h"
#include "debug.h"

#define BLACK 0,0,0
#define WHITE 255,255,255

#define IMGDIR "images/"
#define SPACE "space.bmp"
#define PLAYER "Ship.bmp"
#define PLAYER2 "Ship2.bmp"
#define PLAYER3 "Ship3.bmp"
#define FORWARD_N "Forward Shooter-N.bmp"
#define FORWARD_E "Forward Shooter-E.bmp"
#define FORWARD_S "Forward Shooter-S.bmp"
#define FORWARD_W "Forward Shooter-W.bmp"
#define DUAL_V "Dual Shooter-V.bmp"
#define DUAL_H "Dual Shooter-H.bmp"
#define CRATE "Crate.bmp"
#define LASER_N "Laser-N.bmp"
#define LASER_E "Laser-E.bmp"
#define LASER_S "Laser-S.bmp"
#define LASER_W "Laser-W.bmp"
#define ENEMY_LASER_V "EnemyLaser-V.bmp"
#define ENEMY_LASER_H "EnemyLaser-H.bmp"
#define WALL2 "Wall2.bmp"
#define WALL_H "wall-H.bmp"
#define WALL_UR "wall2-UR.bmp"
#define WALL_UL "wall2-UL.bmp"
#define WALL_V "wall-V.bmp"
#define EXPLOSION "Explosion.bmp"
#define CREDITS "Credits.bmp"
#define INSTRUCTIONS "Instructions.bmp"
#define MENU "StartMenu.bmp"
#define MENU_BOX "StartMenuOutline.bmp"
#define BOUNDS_V "bounds-V.bmp"
#define BOUNDS_H "bounds-H.bmp"
#define LEVEL_WIN "LevelComplete.bmp"
#define LEVEL_LOSE "LevelLose.bmp"
#define SHIP_SELECT "ShipSelect.bmp"
#define SHIP_SELECT_BAR "ShipSelectBar.bmp"
#define GAME_OVER "GameOver.bmp"

SDL_Surface *game_screen; //back buffer

/* Images */
SDL_Surface* images[LEN_IMAGES];
bool loaded = false;

#define LOAD(imgnum, filename) \
	images[(imgnum)] = LoadImage((filename)); \
	if (images[(imgnum)] == NULL) return false

bool InitImages() {
	LOAD(IMG_BG_SPACE, IMGDIR SPACE);
	LOAD(IMG_PLAYER, IMGDIR PLAYER);
	LOAD(IMG_PLAYER2, IMGDIR PLAYER2);
	LOAD(IMG_PLAYER3, IMGDIR PLAYER3);
	LOAD(IMG_FORWARD_N, IMGDIR FORWARD_N);
	LOAD(IMG_FORWARD_E, IMGDIR FORWARD_E);
	LOAD(IMG_FORWARD_S, IMGDIR FORWARD_S);
	LOAD(IMG_FORWARD_W, IMGDIR FORWARD_W);
	LOAD(IMG_DUAL_V, IMGDIR DUAL_V);
	LOAD(IMG_DUAL_H, IMGDIR DUAL_H);
	LOAD(IMG_CRATE, IMGDIR CRATE);
	LOAD(IMG_LASER_N, IMGDIR LASER_N);
	LOAD(IMG_LASER_E, IMGDIR LASER_E);
	LOAD(IMG_LASER_S, IMGDIR LASER_S);
	LOAD(IMG_LASER_W, IMGDIR LASER_W);
	LOAD(IMG_ENEMY_LASER_V, IMGDIR ENEMY_LASER_V);
	LOAD(IMG_ENEMY_LASER_H, IMGDIR ENEMY_LASER_H);
	LOAD(IMG_WALL2, IMGDIR WALL2);
	LOAD(IMG_WALL_H, IMGDIR WALL_H);
	LOAD(IMG_WALL_UR, IMGDIR WALL_UR);
	LOAD(IMG_WALL_UL, IMGDIR WALL_UL);
	LOAD(IMG_WALL_V, IMGDIR WALL_V);
	LOAD(IMG_EXPLOSION, IMGDIR EXPLOSION);
	LOAD(IMG_CREDITS, IMGDIR CREDITS);
	LOAD(IMG_INSTRUCTIONS, IMGDIR INSTRUCTIONS);
	LOAD(IMG_MENU, IMGDIR MENU);
	LOAD(IMG_MENU_BOX, IMGDIR MENU_BOX);
	LOAD(IMG_BOUNDS_V, IMGDIR BOUNDS_V);
	LOAD(IMG_BOUNDS_H, IMGDIR BOUNDS_H);
	LOAD(IMG_LEVEL_WIN, IMGDIR LEVEL_WIN);
	LOAD(IMG_LEVEL_LOSE, IMGDIR LEVEL_LOSE);
	LOAD(IMG_SHIP_SELECT, IMGDIR SHIP_SELECT);
	LOAD(IMG_SHIP_SELECT_BAR, IMGDIR SHIP_SELECT_BAR);
	LOAD(IMG_GAME_OVER, IMGDIR GAME_OVER);
	loaded = true;
	db_log(DB_IMAGE, "Loaded images.\n");
	return true;
}

void UnloadImages() {
	for (int i = 0; i < LEN_IMAGES; i++) {
		SDL_FreeSurface(images[i]);
	}
	loaded = false;
}

SDL_Surface* GetImage(int image) {
	if (!loaded) {
		InitImages();
	}
	return images[image];
}

//I have set the flag SDL_SWSURFACE for a window :)
SDL_Surface* InitVideo(Uint32 flags) {
	// Load SDL
	if (SDL_Init(SDL_INIT_VIDEO) != 0) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		return NULL;
	}
	atexit(SDL_Quit);
 
	// fullscreen can be toggled at run time
	//set the main screen to SCREEN_WIDTHxSCREEN_HEIGHT with a color depth of 16:
	game_screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 16, flags);
	if (game_screen == NULL) {
		fprintf(stderr, "Unable to set video mode: %s\n", SDL_GetError());
		return NULL;
	}

	SDL_WM_SetCaption("Backfire","Backfire");

	return game_screen;
}

void Slock(SDL_Surface *screen) {
	if (SDL_MUSTLOCK(screen)) {
		if (SDL_LockSurface(screen) < 0) {
			return;
		}
	}
}

void Sulock(SDL_Surface *screen) {
	if (SDL_MUSTLOCK(screen)) {
		SDL_UnlockSurface(screen);
	}
}

void DrawPixel(int x, int y,
			unsigned char R, unsigned char G, unsigned char B) {
	Uint32 color = SDL_MapRGB(game_screen->format, R, G, B);
	memcpy((char*)game_screen->pixels + (y*game_screen->pitch) +
			(x*game_screen->format->BytesPerPixel),
					&color, game_screen->format->BytesPerPixel);
}

SDL_Surface* LoadImage(const char *imgname) {
	SDL_Surface *image;
	SDL_Surface *raw;
 
	raw = SDL_LoadBMP(imgname);
	SDL_SetColorKey(raw, SDL_SRCCOLORKEY | SDL_RLEACCEL,
			SDL_MapRGB(raw->format, WHITE));
	if (raw == NULL) {
		fprintf(stderr, "Error loading %s: %s\n", imgname, SDL_GetError());
		return NULL;
	}
	image = SDL_DisplayFormat(raw);
	SDL_FreeSurface(raw);
	return image;
}

void DrawImage(SDL_Surface *image, int x, int y) {
	DrawPartialImage(image, x, y, 0, 0, image->w, image->h);
}

void DrawPartialImage(SDL_Surface *image, int x, int y, int srcx, int srcy,
				int srcw, int srch) {
	SDL_Rect src, dest;
 
	/* Portion of image to display */
	src.x = srcx;
	src.y = srcy;
	src.w = srcw;
	src.h = srch;

	/* Display image from (x,y) */
	dest.x = x;
	dest.y = y;
	dest.w = srcw;
	dest.h = srch;
 
	SDL_BlitSurface(image, &src, game_screen, &dest);
}
